@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Mark Hills <mark@xwax.org>
+ * Copyright (C) 2014 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,11 +25,16 @@
 
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(*x))
 
-void controller_init(struct controller *c, struct controller_ops *ops)
+int controller_init(struct controller *c, struct controller_ops *ops,
+                    void *local, struct rt *rt)
 {
     debug("%p", c);
+
     c->fault = false;
     c->ops = ops;
+    c->local = local;
+
+    return rt_add_controller(rt, c);
 }
 
 void controller_clear(struct controller *c)
