@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@xwax.org>
+ * Copyright (C) 2014 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  */
 
 #include <assert.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,7 +27,7 @@
 #include "debug.h"
 #include "timecoder.h"
 
-#define ZERO_THRESHOLD 128
+#define ZERO_THRESHOLD (128 << 16)
 
 #define ZERO_RC 0.001 /* time constant for zero/rumble filter */
 
@@ -289,7 +290,7 @@ static void init_channel(struct timecoder_channel *ch)
  */
 
 void timecoder_init(struct timecoder *tc, struct timecode_def *def,
-                    double speed, unsigned int sample_rate)
+                    double speed, unsigned int sample_rate, bool phono)
 {
     assert(def != NULL);
 
@@ -302,13 +303,16 @@ void timecoder_init(struct timecoder *tc, struct timecode_def *def,
 
     tc->dt = 1.0 / sample_rate;
     tc->zero_alpha = tc->dt / (ZERO_RC + tc->dt);
+    tc->threshold = ZERO_THRESHOLD;
+    if (phono)
+        tc->threshold >>= 5; /* approx -36dB */
 
     tc->forwards = 1;
     init_channel(&tc->primary);
     init_channel(&tc->secondary);
     pitch_init(&tc->pitch, tc->dt);
 
-    tc->ref_level = 32768.0;
+    tc->ref_level = INT_MAX;
     tc->bitstream = 0;
     tc->timecode = 0;
     tc->valid_counter = 0;
@@ -365,16 +369,17 @@ void timecoder_monitor_clear(struct timecoder *tc)
  */
 
 static void detect_zero_crossing(struct timecoder_channel *ch,
-                                 signed int v, double alpha)
+                                 signed int v, double alpha,
+                                 signed int threshold)
 {
     ch->crossing_ticker++;
 
     ch->swapped = false;
-    if (v > ch->zero + ZERO_THRESHOLD && !ch->positive) {
+    if (v > ch->zero + threshold && !ch->positive) {
         ch->swapped = true;
         ch->positive = true;
         ch->crossing_ticker = 0;
-    } else if (v < ch->zero - ZERO_THRESHOLD && ch->positive) {
+    } else if (v < ch->zero - threshold && ch->positive) {
         ch->swapped = true;
         ch->positive = false;
         ch->crossing_ticker = 0;
@@ -389,31 +394,35 @@ static void detect_zero_crossing(struct timecoder_channel *ch,
 
 static void update_monitor(struct timecoder *tc, signed int x, signed int y)
 {
-    int px, py, p;
-    double v, w;
+    int px, py, size, ref;
 
     if (!tc->mon)
         return;
 
+    size = tc->mon_size;
+    ref = tc->ref_level;
+
     /* Decay the pixels already in the montior */
 
     if (++tc->mon_counter % MONITOR_DECAY_EVERY == 0) {
-        for (p = 0; p < SQ(tc->mon_size); p++) {
+        int p;
+
+        for (p = 0; p < SQ(size); p++) {
             if (tc->mon[p])
                 tc->mon[p] = tc->mon[p] * 7 / 8;
         }
     }
 
-    v = (double)x / tc->ref_level / 2;
-    w = (double)y / tc->ref_level / 2;
+    assert(ref > 0);
 
-    px = tc->mon_size / 2 + (v * tc->mon_size / 2);
-    py = tc->mon_size / 2 + (w * tc->mon_size / 2);
+    /* ref_level is half the prevision of signal level */
+    px = size / 2 + (long long)x * size / ref / 8;
+    py = size / 2 + (long long)y * size / ref / 8;
 
-    /* Set the pixel value to white */
+    if (px < 0 || px >= size || py < 0 || py >= size)
+        return;
 
-    if (px > 0 && px < tc->mon_size && py > 0 && py < tc->mon_size)
-        tc->mon[py * tc->mon_size + px] = 0xff;
+    tc->mon[py * size + px] = 0xff; /* white */
 }
 
 /*
@@ -458,7 +467,8 @@ static void process_bitstream(struct timecoder *tc, signed int m)
 
     /* Adjust the reference level based on this new peak */
 
-    tc->ref_level = (tc->ref_level * (REF_PEAKS_AVG - 1) + m) / REF_PEAKS_AVG;
+    tc->ref_level -= tc->ref_level / REF_PEAKS_AVG;
+    tc->ref_level += m / REF_PEAKS_AVG;
 
     debug("%+6d zero, %+6d (ref %+6d)\t= %d%c (%5d)",
           tc->primary.zero,
@@ -469,17 +479,16 @@ static void process_bitstream(struct timecoder *tc, signed int m)
 
 /*
  * Process a single sample from the incoming audio
+ *
+ * The two input signals (primary and secondary) are in the full range
+ * of a signed int; ie. 32-bit signed.
  */
 
 static void process_sample(struct timecoder *tc,
 			   signed int primary, signed int secondary)
 {
-    signed int m; /* pcm sample, sum of two shorts */
-
-    detect_zero_crossing(&tc->primary, primary, tc->zero_alpha);
-    detect_zero_crossing(&tc->secondary, secondary, tc->zero_alpha);
-
-    m = abs(primary - tc->primary.zero);
+    detect_zero_crossing(&tc->primary, primary, tc->zero_alpha, tc->threshold);
+    detect_zero_crossing(&tc->secondary, secondary, tc->zero_alpha, tc->threshold);
 
     /* If an axis has been crossed, use the direction of the crossing
      * to work out the direction of the vinyl */
@@ -522,6 +531,10 @@ static void process_sample(struct timecoder *tc,
     if (tc->secondary.swapped &&
        tc->primary.positive == ((tc->def->flags & SWITCH_POLARITY) == 0))
     {
+        signed int m;
+
+        /* scale to avoid clipping */
+        m = abs(primary / 2 - tc->primary.zero / 2);
 	process_bitstream(tc, m);
     }
 
@@ -562,24 +575,29 @@ void timecoder_cycle_definition(struct timecoder *tc)
 
 /*
  * Submit and decode a block of PCM audio data to the timecode decoder
+ *
+ * PCM data is in the full range of signed short; ie. 16-bit signed.
  */
 
 void timecoder_submit(struct timecoder *tc, signed short *pcm, size_t npcm)
 {
     while (npcm--) {
-	signed int primary, secondary;
+	signed int left, right, primary, secondary;
+
+        left = pcm[0] << 16;
+        right = pcm[1] << 16;
 
         if (tc->def->flags & SWITCH_PRIMARY) {
-            primary = pcm[0];
-            secondary = pcm[1];
+            primary = left;
+            secondary = right;
         } else {
-            primary = pcm[1];
-            secondary = pcm[0];
+            primary = right;
+            secondary = left;
         }
 
 	process_sample(tc, primary, secondary);
+        update_monitor(tc, left, right);
 
-        update_monitor(tc, pcm[0], pcm[1]);
         pcm += TIMECODER_CHANNELS;
     }
 }
@@ -600,15 +618,15 @@ signed int timecoder_get_position(struct timecoder *tc, double *when)
 {
     signed int r;
 
-    if (tc->valid_counter > VALID_BITS) {
-        r = lut_lookup(&tc->def->lut, tc->bitstream);
+    if (tc->valid_counter <= VALID_BITS)
+        return -1;
 
-        if (r >= 0) {
-            if (when)
-                *when = tc->timecode_ticker * tc->dt;
-            return r;
-        }
-    }
+    r = lut_lookup(&tc->def->lut, tc->bitstream);
+    if (r == -1)
+        return -1;
 
-    return -1;
+    if (when)
+        *when = tc->timecode_ticker * tc->dt;
+
+    return r;
 }
